/*
 * Copyright (c) 2007, 2008, 2009, 2010, 2011 David Berkman
 * 
 * This file is part of the SmallMind Code Project.
 * 
 * The SmallMind Code Project is free software, you can redistribute
 * it and/or modify it under the terms of GNU Affero General Public
 * License as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 * 
 * The SmallMind Code Project is distributed in the hope that it will
 * be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the the GNU Affero General Public
 * License, along with The SmallMind Code Project. If not, see
 * <http://www.gnu.org/licenses/>.
 * 
 * Additional permission under the GNU Affero GPL version 3 section 7
 * ------------------------------------------------------------------
 * If you modify this Program, or any covered work, by linking or
 * combining it with other code, such other code is not for that reason
 * alone subject to any of the requirements of the GNU Affero GPL
 * version 3.
 */
package org.smallmind.swing;

import java.awt.Color;
import java.security.SecureRandom;
import java.util.HashMap;
import java.util.Random;

public class Colorizer {

  private static final Random RANDOM = new SecureRandom();
  private static final double GOLDEN_RATIO_CONJUGATE = 1 / ((1 + Math.sqrt(5)) / 2);

  private final HashMap<String, Color> colorMap = new HashMap<String, Color>();

  private double hue = RANDOM.nextDouble();

  public synchronized Color getColor (String name) {

    Color color;

    if ((color = colorMap.get(name)) == null) {

      hue += GOLDEN_RATIO_CONJUGATE;
      hue %= 1;

      colorMap.put(name, color = Color.getHSBColor((float)hue, (RANDOM.nextInt(50) + 50) / 100F, (RANDOM.nextInt(25) + 75) / 100F));
    }

    return color;
  }
}
